/* mkPmtIstFc-3.cc */
/* originally coded by Yutaka Ichikawa */
/* Tue Nov 15 2005 modified by Yasuto Tachikawa */

// gcc v 3.4 ŃRpCł悤ɂB by YT 2007.11.26

/*
[Diffference from mkPmtIstFc-2.c]
1) Discharge at the outlet and catchment area are input for setting initial condition.
2) Two types of land use (ex. forest and river) are possible to set.
*/

/*
[Program name]

mkPmtIstFc-3.cc

[Version no.]

ver. 1.1

[Function of the program]

The parameters of Q-H relationship developed by Tachikawa et al. 2004, 
Annual Journal of Hydraulic Eng., JSCE, vol. 45 and initial condition
of slope edges are set using the program. Outputs are two files:

 - parameter file (KsEdgeFc Parameter Format)
 - initial discharge file (KsEdgeFc Initial State Format)

Input data are:

 - edge connection V.1, V.2, V.3, V.4, or V.5
 - default parameter templete file (e.g. forest)
 - 2nd land use parameter file (e.g. river)


܂ł̃vOƂ̈Ⴂ́Aԗ (eGbW㉺[ł̗) 
ݒ̎dłB̓t@CƂ edge connection V.1, V.2, V.4 ǂݍ
ꍇ́A܂Œʂ㉺[̗ʂW͂͂AׂẴGbW㉺
[ŗʂɂȂ܂Bedge connection V.3, V.5 ǂݍ񂾏ꍇɂ́AvZ
Jn̗o (mm/hour) ^āAeGbW㉺[ɂԂ̗
vZAԗʂƂ܂BƂ΁AvZJn̗o r, 
GbW̏㗬̖ʐς Au, ̃GbW̖ʐς Ac ƂƁA
GbW̏[ł̏ Qu, [ł̏ Qc ͂ꂼA

  Qu = r * Au
  Qc = r * (Au + Ac)

ƂȂ܂B

[How to use]

% mkPmtIstFc-3.exe edgeConnectionV5.dat paramete.dat parameter1.dat area.pmt area.ist

edgeConnectionV5.dat:   input. Edge connection V5 data
parameter.dat : input. parameter value templete file for default land use (eg. forest).
parameter1.dat : input. parameter value templete file for another land use (eg. river).
area.pmt               : output. parameter file to be used with ksEdgeFc2d.
area.ist       :       output. initial file to be used with ksEdgeFc2d.


Following is an example of shell script to process many files at the same time.

---
#!/usr/bin/sh

mkPmtIstFc-3.exe edgeConnectionArea.dat parameterLanduse.dat parameterLanduse1.dat area.pmt area.ist
---

An example of parameter.dat is as follows: Description after # is not valid for input file.
Please do not include # and later comments.

0.3       # manning coefficient (m-s) n
1.667     # kinematic parameter (-) m
0.01      # hydraulic conductivity (m/s), ka
1.0       # depth of soil layer (m), D
0.55      # depth of A-layer (m), da
0.45      # depth of field capacity (m), dm
4.0       # beta [> 1.0] (-)
0.65      # theta [0.65] (-)
0.0000001 # minimum value of C = dQ/dA [0.000001] (m/s)
10        # Qlevel [10] (mm/hour)
1.00      # eps [0.0001] (-)
10        # number of spatial division (-)
1.00      # initial runoff height (mm/hour)


Last two lines is valid when reading edge connection V.3, V.5 file.
To read edge connection V.1, V.2, V.4 file,

0.0       # discharge at upper end (m3/s)
0.0       # discharge at lower end (m3/s)


An example of parameter1.dat is as follows: Description after # is not valid for input file.
Please do not include # and later comments.

0.1       # manning coefficient (m-s) n
1.667     # kinematic parameter (-) m
0.01      # hydraulic conductivity (m/s), ka
1.0       # depth of soil layer (m), D
0.0       # depth of A-layer (m), da
0.0       # depth of field capacity (m), dm
4.0       # beta [> 1.0] (-)
0.65      # theta [0.65] (-)
0.0000001 # minimum value of C = dQ/dA [0.000001] (m/s)
10        # Qlevel [10] (mm/hour)
1.00      # eps [0.0001] (-)
10        # number of spatial division (-)
50        # number of edge having the value of parameterLanduse2.dat
22, 25, .... # edge number having the value of parameterLanduse2.dat
             # In the case, 50 edge numbers are specified here.

[Input edge connection file]

edge connection V.1, V.2, V.3, V.4, V.5

[RpC]

% make

*/

#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#include "khboolean.h"
#include "khsimple.h"
#include "kherror.h"
#include "khstr.h"
#include "khdfstream.h"
#include "nopath.h"

using namespace std;

void usage(const KhString& command)
{
  cerr
  << "\n" << command << " produces KsEdgeFc Parameter Format data and KsEdgeFc Initial State data\n"
  << "usage: " << command << " [options]\n"
  << "  options: \n"
  << "    -h  show this help message\n"
  << "    -v  show version number\n"
  << "  iFile1 : edge connection V.1, V.2, V.3, V.4 or V.5 data\n"
  << "  iFile2 : parameter tenplete file for default land use (e.g. forest)\n"
  << "  iFile3 : parameter tenplete file for another land use (e.g. river)\n"
  << "  oFile1 : KsEdgeFc Parameter Format data\n"
  << "  oFile2 : KsEdgeFc Initial State data\n"
  << "  \"iFile\", \"oFile1\" and \"oFile2\" will be read in from STDIN\n"
  << "  with prompting messages.\n" << endl;
  exit(1);
}

void version(void)
{
  cerr << "\nThis is mkPmtIstFc ver. 1.0\n"
       << "developed by Yutaka ICHIKAWA, Kyoto Univ.\n" << endl;
}

int main						// CvO
(
 int argc,
 char** argv
 )
{
  KhString command = nopath(argv[0]);
  
  // optstring ŁAIvVƂċw肵܂B
  char *optstring = "hv";
  
  int c;
  while((c = getopt(argc, argv, optstring)) != EOF)
    {
      // getopt ͉͂IvVԂ܂B
      switch (c) {
      case 'v':
	version();
	break;
      case 'h':
	// IvV h ԂĂꍇ̏
	// fall through
      default:
	usage(command);
      }
    }
  
  // argc ɂ, R}hƈ̌܂BȂ
  // ƂCargc ɂ 1 ܂B
  
  if (argc != 6)
    {
      //  command ł́Ǎ0łB
      usage(command);
    }
  
  char in[256], out[256], out2[256], parain[256], parain1[256];
  double man, man1;
  double m, m1;
  double ka, ka1;
  double D, D1;
  double da, da1;
  double dm, dm1;
  double beta, beta1;
  double theta, theta1;
  double cmin, cmin1;
  double Qlevel, Qlevel1;
  double eps, eps1;
  int ndiv, ndiv1;
  FILE *iFile2, *iFile3;
  
//  KhBoolean ans = NO;
//  char wbuf[256];
  
  strcpy(in, argv[1]); // setting file name for edge connection data.
  strcpy(parain, argv[2]);  // setting file name for default land use.
  strcpy(parain1, argv[3]); // setting file name for other land use.
  
  KhDfstream iFile(in);
  if (!iFile)
    {
      cerr << "E: no such file." << endl;
      exit(1);
    }

  cout << "Processing " << argv[1] << endl;

  if ((iFile2 = fopen(parain, "r")) == NULL) {
    cout << "File cannot open (parameter Land use 1)!\n";
    exit(1);
  }
  
  if ((iFile3 = fopen(parain1, "r")) == NULL) {
    cout << "File cannot open (parameter Land use 2)!\n";
    exit(1);
  }

  KhString buf;
  //	while (getline(iFile, buf) && buf != "[Data Format Name]") ;
  while (getline(iFile, buf) && 
	 strncmp((const char *)buf, "[Data Format Name]", 18) != 0) ;
  getline(iFile, buf);
  // 	if (buf != "edge connection V.1" && buf != "edge connection V.2" &&
  // 		buf != "edge connection V.3")
  if (strncmp((const char *)buf, "edge connection V.1", 19) != 0 && 
      strncmp((const char *)buf, "edge connection V.2", 19) != 0 &&
      strncmp((const char *)buf, "edge connection V.3", 19) != 0 &&
      strncmp((const char *)buf, "edge connection V.4", 19) != 0 &&
      strncmp((const char *)buf, "edge connection V.5", 19) != 0)
    {
      cerr << "E: invalid data format." << endl;
      exit(1);
    }
  
  // f[^tH[}bg̎
  int topoDataType;
  // 	if (buf == "edge connection V.1") topoDataType = 1;
  // 	else if (buf == "edge connection V.2") topoDataType = 2;
  // 	else topoDataType = 3;
  
  if (strncmp((const char *)buf, "edge connection V.1", 19) == 0)
    topoDataType = 1;
  else if (strncmp((const char *)buf, "edge connection V.2", 19) == 0)
    topoDataType = 2;
  else if (strncmp((const char *)buf, "edge connection V.3", 19) == 0)
    topoDataType = 3;
  else if (strncmp((const char *)buf, "edge connection V.4", 19) == 0)
    topoDataType = 4;
  else
    topoDataType = 5;

  strcpy(out, argv[4]);
  strcpy(out2, argv[5]);

  fscanf(iFile2, "%lf", &man);
  fscanf(iFile2, "%lf", &m);
  fscanf(iFile2, "%lf", &ka);
  fscanf(iFile2, "%lf", &D);
  fscanf(iFile2, "%lf", &da);
  fscanf(iFile2, "%lf", &dm);
  fscanf(iFile2, "%lf", &beta);
  fscanf(iFile2, "%lf", &theta);
  fscanf(iFile2, "%lf", &cmin);
  fscanf(iFile2, "%lf", &Qlevel);
  fscanf(iFile2, "%lf", &eps);
  fscanf(iFile2, "%d", &ndiv);

  fscanf(iFile3, "%lf", &man1);
  fscanf(iFile3, "%lf", &m1);
  fscanf(iFile3, "%lf", &ka1);
  fscanf(iFile3, "%lf", &D1);
  fscanf(iFile3, "%lf", &da1);
  fscanf(iFile3, "%lf", &dm1);
  fscanf(iFile3, "%lf", &beta1);
  fscanf(iFile3, "%lf", &theta1);
  fscanf(iFile3, "%lf", &cmin1);
  fscanf(iFile3, "%lf", &Qlevel1);
  fscanf(iFile3, "%lf", &eps1);
  fscanf(iFile3, "%d", &ndiv1);

  int nedge1, iland;
  fscanf(iFile3, "%d", &nedge1);
  int landuse1[nedge1];
  
  for (int i=1; i <= nedge1; i++) { // read edge number of landuse2
      fscanf(iFile3, "%d", &iland);
      landuse1[i] = iland;
  }
  
  // ԗʂ̓
  
  double updis, lowdis;	   // GbW㉺[ł̗
  double irunoff;
  
  if (topoDataType == 1 || topoDataType == 2 || topoDataType == 4)
    {
	  fscanf(iFile2, "%lf", &updis);
	  fscanf(iFile2, "%lf", &lowdis);
    }
  else
    {
	  fscanf(iFile2, "%lf", &irunoff);
   }
  
 
  ofstream oFile(out);
  ofstream oFile2(out2);
  
  oFile << "[Comment]\n" 
	<< "This data was generated from " << in 
	<< " by " << command << ".\n"
	<< "Each column in [Data] section is as follows:\n"
	<< "<edge number>\n<manning coefficient (m-s)>\n"
	<< "<kinematic parameter m (-)>\n"
	<< "<hydraulic conductivity (m/s)>\n"
	<< "<depth of soil layer (m)>\n"
	<< "<depth of A-layer (m)>\n"
	<< "<depth of soil matrix layer (m)>\n"
	<< "<beta (-)>\n"
	<< "<theta>\n<minimun C = dQ/dA(m/s)>\n"
	<< "<Qlevel(mm/hour)>\n<eps(-)>\n"
	<< "<number of spatial division(-)>"
	<< endl;
  
  oFile << "[Data Format Name]\n"
	<< "KsEdgeFc Parameter Format" << endl;
  
  oFile2 << "[Comment]\n"
	 << "This data was generated from " << in 
	 << " by " << command << ".\n"
	 << "Each column in [Data] section is as follows:\n"
	 << "<edge number>\n"
	 << "<discharge at upper end(m3/s)>\n"
	 << "<discharge at lower end(m3/s)>"
	 << endl;
  
  oFile2 << "[Data Format Name]\n"
	 << "KsEdgeFc Initial State Format" << endl;
  
  //	while (getline(iFile, buf) && buf != "[Number of Total Edges]") ;
  while (getline(iFile, buf) && 
	 strncmp((const char *)buf, "[Number of Total Edges]", 23) != 0) ;
  
  long numberOfTotalEdges;
  getline(iFile, buf);

  // istrstream strmbuf(buf);
  istringstream strmbuf((const char*)buf);

  strmbuf >> numberOfTotalEdges;
  oFile << "[Number of Total Edges]\n" << numberOfTotalEdges << endl;
  oFile2 << "[Number of Total Edges]\n" << numberOfTotalEdges << endl;
  
  //	while (getline(iFile, buf) && buf != "[Data]") ;
  while (getline(iFile, buf) && 
	 strncmp((const char *)buf, "[Data]", 6) != 0) ;
  
  oFile << "[Data]" << endl;
  oFile2 << "[Data]" << endl;
  
  while (getline(iFile, buf)) 
    {
      long num;				   // GbWԍ
      double x, y, z;			   // _̍W(sv)
      long upperNode, lowerNode; // GbW㉺[̃m[hԍ
      double upx, upy, upz, lowx, lowy, lowz;	// GbW㉺[̍W(sv)
      double upnum, lownum, tmp; // ㉺GbW̐Aԍ(sv)
      int atr;				   // 
      double len, grd, area;	   // GbW̒AzAʐ
      double dist, upArea;	   // ʕzA㗬̖ʐ
      
      //istrstream strmdata(buf);
      istringstream strmdata((const char*)buf);

      strmdata >> num ;
      
      if (topoDataType == 2 || topoDataType == 3)
	strmdata >> x >> y >> z ;
      else if (topoDataType == 4 || topoDataType == 5)
	strmdata >> upperNode >> upx >> upy >> upz 
		 >> lowerNode >> lowx >> lowy >> lowz ;
      
      strmdata >> upnum ;
      for (int i = 0; i < upnum; i++) strmdata >> tmp ;
      strmdata >> lownum ;
      for (int i = 0; i < lownum; i++) strmdata >> tmp ;
      strmdata >> atr ;
      
      if (atr == 1)			// _-_GbW
	{
	  strmdata >> len >> grd >> area ;
	}
      else if (atr == 2)		// _-͓_GbW
	{
	  len = grd = area = 0.0;
	}
      else
	{
	  strmdata >> len >> grd ;
	  area = 0.0;
	}
      
      if (topoDataType == 3 || topoDataType == 5)
	strmdata >> dist >> upArea ;
      
      // setting parameter
      
      int landUseType=0;
      for (int i=1; i <= nedge1; i++) { // check landuse type.
        if (num == landuse1[i]) {
           landUseType = 1;
            break;
        }
      }
      
      if (landUseType == 1) { // set parameter for land use type 1
        oFile << num << " " << man1 << " " << m1 << " " 
              << ka1 << " " << D1 << " " << da1 << " " 
              << dm1 << " " << beta1 << " "
              << theta1 << " " << cmin1 << " " << Qlevel1 << " " 
              << eps1 << " " << ndiv << endl;
      }
      else {                  // set parameter default for land use type
        oFile << num << " " << man << " " << m << " " 
              << ka << " " << D << " " << da << " " 
              << dm << " " << beta << " "
              << theta << " " << cmin << " " << Qlevel << " " 
              << eps << " " << ndiv << endl;
      }

      // ԗ [m3/s]
      if (topoDataType == 1 || topoDataType == 2 || topoDataType == 4)
	{
	  oFile2 << num << " " << updis << " " << lowdis << endl;
	}
      else
	{
	  oFile2 << num << " " << irunoff * upArea * 0.001 / 3600.0 << " " 
		 << irunoff * (upArea + area) * 0.001 / 3600.0 << endl;
	}
    }
  
  return 0;
}
